@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// Copyright 2020 Authors of Cilium
+// Copyright Authors of Cilium
 
 package clustermesh
 
@@ -7,10 +7,12 @@ import (
 	"bytes"
 	"context"
 	"encoding/base64"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
 	"net"
+	"regexp"
 	"strconv"
 	"strings"
 	"text/tabwriter"
@@ -23,11 +25,14 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/intstr"
 
+	"github.com/blang/semver/v4"
 	"github.com/cilium/cilium/api/v1/models"
 	ciliumv2 "github.com/cilium/cilium/pkg/k8s/apis/cilium.io/v2"
+	"github.com/cilium/cilium/pkg/versioncheck"
 
 	"github.com/cilium/cilium-cli/defaults"
 	"github.com/cilium/cilium-cli/internal/certs"
+	"github.com/cilium/cilium-cli/internal/helm"
 	"github.com/cilium/cilium-cli/internal/utils"
 	"github.com/cilium/cilium-cli/k8s"
 	"github.com/cilium/cilium-cli/status"
@@ -227,8 +232,10 @@ func (k *K8sClusterMesh) generateDeployment(clustermeshApiserverArgs []string) *
 								"--trusted-ca-file=/var/lib/etcd-secrets/ca.crt",
 								"--cert-file=/var/lib/etcd-secrets/tls.crt",
 								"--key-file=/var/lib/etcd-secrets/tls.key",
-								"--listen-client-urls=https://127.0.0.1:2379,https://$(HOSTNAME_IP):2379",
-								"--advertise-client-urls=https://$(HOSTNAME_IP):2379",
+								// Surrounding the IPv4 address with brackets works in this case, since etcd
+								// uses net.SplitHostPort() internally and it accepts that format.
+								"--listen-client-urls=https://127.0.0.1:2379,https://[$(HOSTNAME_IP)]:2379",
+								"--advertise-client-urls=https://[$(HOSTNAME_IP)]:2379",
 								"--initial-cluster-token=clustermesh-apiserver",
 								"--auto-compaction-retention=1",
 							},
@@ -426,6 +433,8 @@ type k8sClusterMeshImplementation interface {
 	GetRunningCiliumVersion(ctx context.Context, namespace string) (string, error)
 	ListCiliumEndpoints(ctx context.Context, namespace string, options metav1.ListOptions) (*ciliumv2.CiliumEndpointList, error)
 	GetPlatform(ctx context.Context) (*k8s.Platform, error)
+	CiliumLogs(ctx context.Context, namespace, pod string, since time.Time, filter *regexp.Regexp) (string, error)
+	GetHelmState(ctx context.Context, namespace string, secretName string) (*helm.State, error)
 }
 
 type K8sClusterMesh struct {
@@ -459,6 +468,8 @@ type Parameters struct {
 	All                  bool
 	ConfigOverwrites     []string
 	Retries              int
+	HelmValuesSecretName string
+	Output               string
 }
 
 func (p Parameters) validateParams() error {
@@ -620,16 +631,16 @@ func (k *K8sClusterMesh) Enable(ctx context.Context) error {
 }
 
 type accessInformation struct {
-	ServiceIPs           []string
-	ServicePort          int
-	ClusterID            string
-	ClusterName          string
-	CA                   []byte
-	ClientCert           []byte
-	ClientKey            []byte
-	ExternalWorkloadCert []byte
-	ExternalWorkloadKey  []byte
-	Tunnel               string
+	ServiceIPs           []string `json:"service_ips,omitempty"`
+	ServicePort          int      `json:"service_port,omitempty"`
+	ClusterID            string   `json:"cluster_id,omitempty"`
+	ClusterName          string   `json:"cluster_name,omitempty"`
+	CA                   []byte   `json:"ca,omitempty"`
+	ClientCert           []byte   `json:"client_cert,omitempty"`
+	ClientKey            []byte   `json:"client_key,omitempty"`
+	ExternalWorkloadCert []byte   `json:"external_workload_cert,omitempty"`
+	ExternalWorkloadKey  []byte   `json:"external_workload_key,omitempty"`
+	Tunnel               string   `json:"tunnel,omitempty"`
 }
 
 func (ai *accessInformation) etcdConfiguration() string {
@@ -649,7 +660,46 @@ func (ai *accessInformation) validate() bool {
 		ai.ClusterID != "0"
 }
 
-func (k *K8sClusterMesh) extractAccessInformation(ctx context.Context, client k8sClusterMeshImplementation, endpoints []string, verbose bool) (*accessInformation, error) {
+func getDeprecatedName(secretName string) string {
+	switch secretName {
+	case defaults.ClusterMeshServerSecretName,
+		defaults.ClusterMeshAdminSecretName,
+		defaults.ClusterMeshClientSecretName,
+		defaults.ClusterMeshExternalWorkloadSecretName:
+		return secretName + "s"
+	default:
+		return ""
+	}
+}
+
+// We had inconsistency in naming clustermesh secrets between Helm installation and Cilium CLI installation
+// Cilium CLI was naming clustermesh secrets with trailing 's'. eg. 'clustermesh-apiserver-client-certs' instead of `clustermesh-apiserver-client-cert`
+// This caused Cilium CLI 'clustermesh status' command to fail when Cilium is installed using Helm
+// getSecret handles both secret names and logs warning if deprecated secret name is found
+func (k *K8sClusterMesh) getSecret(ctx context.Context, client k8sClusterMeshImplementation, secretName string) (*corev1.Secret, error) {
+
+	secret, err := client.GetSecret(ctx, k.params.Namespace, secretName, metav1.GetOptions{})
+	if err != nil {
+		deprecatedSecretName := getDeprecatedName(secretName)
+		if deprecatedSecretName == "" {
+			return nil, fmt.Errorf("unable to get secret %q: %w", secretName, err)
+		}
+
+		k.Log("Trying to get secret %s by deprecated name %s", secretName, deprecatedSecretName)
+
+		secret, err = client.GetSecret(ctx, k.params.Namespace, deprecatedSecretName, metav1.GetOptions{})
+		if err != nil {
+			return nil, fmt.Errorf("unable to get secret %q: %w", deprecatedSecretName, err)
+		}
+
+		k.Log("⚠️ Deprecated secret name %q, should be changed to %q", secret.Name, secretName)
+
+	}
+
+	return secret, err
+}
+
+func (k *K8sClusterMesh) extractAccessInformation(ctx context.Context, client k8sClusterMeshImplementation, endpoints []string, verbose bool, getExternalWorkLoadSecret bool) (*accessInformation, error) {
 	cm, err := client.GetConfigMap(ctx, k.params.Namespace, defaults.ConfigMapName, metav1.GetOptions{})
 	if err != nil {
 		return nil, fmt.Errorf("unable to retrieve ConfigMap %q: %w", defaults.ConfigMapName, err)
@@ -683,34 +733,38 @@ func (k *K8sClusterMesh) extractAccessInformation(ctx context.Context, client k8
 		return nil, fmt.Errorf("secret %q does not contain CA cert %q", defaults.CASecretName, defaults.CASecretCertName)
 	}
 
-	meshSecret, err := client.GetSecret(ctx, k.params.Namespace, defaults.ClusterMeshClientSecretName, metav1.GetOptions{})
+	meshSecret, err := k.getSecret(ctx, client, defaults.ClusterMeshClientSecretName)
 	if err != nil {
-		return nil, fmt.Errorf("unable to get secret %q to access clustermesh service: %s", defaults.ClusterMeshClientSecretName, err)
+		return nil, fmt.Errorf("unable to get client secret to access clustermesh service: %w", err)
 	}
 
 	clientKey, ok := meshSecret.Data[corev1.TLSPrivateKeyKey]
 	if !ok {
-		return nil, fmt.Errorf("secret %q does not contain key %q", defaults.ClusterMeshClientSecretName, corev1.TLSPrivateKeyKey)
+		return nil, fmt.Errorf("secret %q does not contain key %q", meshSecret.Name, corev1.TLSPrivateKeyKey)
 	}
 
 	clientCert, ok := meshSecret.Data[corev1.TLSCertKey]
 	if !ok {
-		return nil, fmt.Errorf("secret %q does not contain key %q", defaults.ClusterMeshClientSecretName, corev1.TLSCertKey)
+		return nil, fmt.Errorf("secret %q does not contain key %q", meshSecret.Name, corev1.TLSCertKey)
 	}
 
-	externalWorkloadSecret, err := client.GetSecret(ctx, k.params.Namespace, defaults.ClusterMeshExternalWorkloadSecretName, metav1.GetOptions{})
-	if err != nil {
-		return nil, fmt.Errorf("unable to get secret %q to access clustermesh service: %s", defaults.ClusterMeshExternalWorkloadSecretName, err)
-	}
+	// ExternalWorkload secret is created by 'clustermesh enable' command, but it isn't created by Helm. We should try to load this secret only when needed
+	var externalWorkloadKey, externalWorkloadCert []byte
+	if getExternalWorkLoadSecret {
+		externalWorkloadSecret, err := k.getSecret(ctx, client, defaults.ClusterMeshExternalWorkloadSecretName)
+		if err != nil {
+			return nil, fmt.Errorf("unable to get external workload secret to access clustermesh service")
+		}
 
-	externalWorkloadKey, ok := externalWorkloadSecret.Data[corev1.TLSPrivateKeyKey]
-	if !ok {
-		return nil, fmt.Errorf("secret %q does not contain key %q", defaults.ClusterMeshExternalWorkloadSecretName, corev1.TLSPrivateKeyKey)
-	}
+		externalWorkloadKey, ok = externalWorkloadSecret.Data[corev1.TLSPrivateKeyKey]
+		if !ok {
+			return nil, fmt.Errorf("secret %q does not contain key %q", externalWorkloadSecret.Namespace, corev1.TLSPrivateKeyKey)
+		}
 
-	externalWorkloadCert, ok := externalWorkloadSecret.Data[corev1.TLSCertKey]
-	if !ok {
-		return nil, fmt.Errorf("secret %q does not contain key %q", defaults.ClusterMeshExternalWorkloadSecretName, corev1.TLSCertKey)
+		externalWorkloadCert, ok = externalWorkloadSecret.Data[corev1.TLSCertKey]
+		if !ok {
+			return nil, fmt.Errorf("secret %q does not contain key %q", externalWorkloadSecret.Namespace, corev1.TLSCertKey)
+		}
 	}
 
 	ai := &accessInformation{
@@ -884,7 +938,7 @@ func (k *K8sClusterMesh) Connect(ctx context.Context) error {
 		return fmt.Errorf("unable to create Kubernetes client to access remote cluster %q: %w", k.params.DestinationContext, err)
 	}
 
-	aiRemote, err := k.extractAccessInformation(ctx, remoteCluster, k.params.DestinationEndpoints, true)
+	aiRemote, err := k.extractAccessInformation(ctx, remoteCluster, k.params.DestinationEndpoints, true, false)
 	if err != nil {
 		k.Log("❌ Unable to retrieve access information of remote cluster %q: %s", remoteCluster.ClusterName(), err)
 		return err
@@ -894,7 +948,7 @@ func (k *K8sClusterMesh) Connect(ctx context.Context) error {
 		return fmt.Errorf("remote cluster has non-unique name (%s) and/or ID (%s)", aiRemote.ClusterName, aiRemote.ClusterID)
 	}
 
-	aiLocal, err := k.extractAccessInformation(ctx, k.client, k.params.SourceEndpoints, true)
+	aiLocal, err := k.extractAccessInformation(ctx, k.client, k.params.SourceEndpoints, true, false)
 	if err != nil {
 		k.Log("❌ Unable to retrieve access information of local cluster %q: %s", k.client.ClusterName(), err)
 		return err
@@ -990,12 +1044,12 @@ func (k *K8sClusterMesh) Disconnect(ctx context.Context) error {
 }
 
 type Status struct {
-	AccessInformation *accessInformation
-	Service           *corev1.Service
-	Connectivity      *ConnectivityStatus
+	AccessInformation *accessInformation  `json:"access_information,omitempty"`
+	Service           *corev1.Service     `json:"service,omitempty"`
+	Connectivity      *ConnectivityStatus `json:"connectivity,omitempty"`
 }
 
-func (k *K8sClusterMesh) statusAccessInformation(ctx context.Context, log bool) (*accessInformation, error) {
+func (k *K8sClusterMesh) statusAccessInformation(ctx context.Context, log bool, getExternalWorkloadSecret bool) (*accessInformation, error) {
 	w := utils.NewWaitObserver(ctx, utils.WaitParameters{Log: func(err error, wait string) {
 		if log {
 			k.Log("⌛ Waiting (%s) for access information: %s", wait, err)
@@ -1004,7 +1058,7 @@ func (k *K8sClusterMesh) statusAccessInformation(ctx context.Context, log bool)
 	defer w.Cancel()
 
 retry:
-	ai, err := k.extractAccessInformation(ctx, k.client, []string{}, false)
+	ai, err := k.extractAccessInformation(ctx, k.client, []string{}, false, getExternalWorkloadSecret)
 	if err != nil && k.params.Wait {
 		if err := w.Retry(err); err != nil {
 			return nil, err
@@ -1052,23 +1106,23 @@ func (k *K8sClusterMesh) waitForDeployment(ctx context.Context) error {
 }
 
 type StatisticalStatus struct {
-	Min int64
-	Avg float64
-	Max int64
+	Min int64   `json:"min,omitempty"`
+	Avg float64 `json:"avg,omitempty"`
+	Max int64   `json:"max,omitempty"`
 }
 
 type ClusterStats struct {
-	Configured int
-	Connected  int
+	Configured int `json:"configured,omitempty"`
+	Connected  int `json:"connected,omitempty"`
 }
 
 type ConnectivityStatus struct {
-	GlobalServices StatisticalStatus
-	Connected      StatisticalStatus
-	Clusters       map[string]*ClusterStats
-	Total          int64
-	NotReady       int64
-	Errors         status.ErrorCountMapMap
+	GlobalServices StatisticalStatus        `json:"global_services,omitempty"`
+	Connected      StatisticalStatus        `json:"connected,omitempty"`
+	Clusters       map[string]*ClusterStats `json:"clusters,omitempty"`
+	Total          int64                    `json:"total,omitempty"`
+	NotReady       int64                    `json:"not_ready,omitempty"`
+	Errors         status.ErrorCountMapMap  `json:"errors,omitempty"`
 }
 
 func (c *ConnectivityStatus) addError(pod, cluster string, err error) {
@@ -1167,7 +1221,7 @@ func (k *K8sClusterMesh) determineStatusConnectivity(ctx context.Context) (*Conn
 		Clusters:       map[string]*ClusterStats{},
 	}
 
-	pods, err := k.client.ListPods(ctx, k.params.Namespace, metav1.ListOptions{LabelSelector: "k8s-app=cilium"})
+	pods, err := k.client.ListPods(ctx, k.params.Namespace, metav1.ListOptions{LabelSelector: defaults.AgentPodSelector})
 	if err != nil {
 		return nil, fmt.Errorf("unable to list cilium pods: %w", err)
 	}
@@ -1211,7 +1265,7 @@ func (k *K8sClusterMesh) Status(ctx context.Context) (*Status, error) {
 	defer cancel()
 
 	s := &Status{}
-	s.AccessInformation, err = k.statusAccessInformation(ctx, true)
+	s.AccessInformation, err = k.statusAccessInformation(ctx, true, false)
 	if err != nil {
 		return nil, err
 	}
@@ -1244,6 +1298,15 @@ func (k *K8sClusterMesh) Status(ctx context.Context) (*Status, error) {
 
 	s.Connectivity, err = k.statusConnectivity(ctx)
 
+	if k.params.Output == status.OutputJSON {
+		jsonStatus, err := json.MarshalIndent(s, "", " ")
+		if err != nil {
+			return nil, fmt.Errorf("failed to marshal status to JSON")
+		}
+		fmt.Println(string(jsonStatus))
+		return s, nil
+	}
+
 	if s.Connectivity != nil {
 		if s.Connectivity.NotReady > 0 {
 			k.Log("⚠️  %d/%d nodes are not connected to all clusters [min:%d / avg:%.1f / max:%d]",
@@ -1341,7 +1404,7 @@ func (k *K8sClusterMesh) DeleteExternalWorkload(ctx context.Context, names []str
 	if count > 0 {
 		k.Log("✅ Removed %d external workload resources.", count)
 	} else {
-		k.Log("ℹ️ No external workload resources to remove.")
+		k.Log("ℹ️  No external workload resources to remove.")
 	}
 	if len(errs) > 0 {
 		return errors.New(strings.Join(errs, ", "))
@@ -1437,7 +1500,7 @@ endpoints:
 - https://clustermesh-apiserver.cilium.io:$CLUSTER_PORT
 EOF
 
-CILIUM_OPTS=" --join-cluster --enable-host-reachable-services --enable-endpoint-health-checking=false"
+CILIUM_OPTS=" --join-cluster %[8]s --enable-endpoint-health-checking=false"
 CILIUM_OPTS+=" --kvstore etcd --kvstore-opt etcd.config=/var/lib/cilium/etcd/config.yaml"
 if [ -n "$HOST_IP" ] ; then
     CILIUM_OPTS+=" --ipv4-node $HOST_IP"
@@ -1553,7 +1616,7 @@ func (k *K8sClusterMesh) WriteExternalWorkloadInstallScript(ctx context.Context,
 	}
 	k.Log("✅ Using image from Cilium DaemonSet: %s", daemonSet.Spec.Template.Spec.Containers[0].Image)
 
-	ai, err := k.statusAccessInformation(ctx, false)
+	ai, err := k.statusAccessInformation(ctx, false, true)
 	if err != nil {
 		return err
 	}
@@ -1578,11 +1641,35 @@ func (k *K8sClusterMesh) WriteExternalWorkloadInstallScript(ctx context.Context,
 		k.params.Retries = 1
 	}
 
+	var ciliumVer semver.Version
+
+	helmState, err := k.client.GetHelmState(ctx, k.params.Namespace, k.params.HelmValuesSecretName)
+	if err != nil {
+		// Try to retrieve version from image tag
+		v, err := k.client.GetRunningCiliumVersion(ctx, k.params.Namespace)
+		if err != nil {
+			return err
+		}
+		ciliumVer, err = utils.ParseCiliumVersion(v)
+		if err != nil {
+			return fmt.Errorf("failed to parse Cilium version %s: %w", v, err)
+		}
+
+	} else {
+		ciliumVer = helmState.Version
+	}
+
+	sockLBOpt := "--bpf-lb-sock"
+	if ciliumVer.LT(versioncheck.MustVersion("1.12.0")) {
+		// Before 1.12, the socket LB was enabled via --enable-host-reachable-services flag
+		sockLBOpt = "--enable-host-reachable-services"
+	}
+
 	fmt.Fprintf(writer, installScriptFmt,
 		daemonSet.Spec.Template.Spec.Containers[0].Image, clusterAddr,
 		configOverwrites,
 		string(ai.CA), string(ai.ExternalWorkloadCert), string(ai.ExternalWorkloadKey),
-		strconv.Itoa(k.params.Retries))
+		strconv.Itoa(k.params.Retries), sockLBOpt)
 	return nil
 }
 
@@ -1614,7 +1701,7 @@ func (k *K8sClusterMesh) ExternalWorkloadStatus(ctx context.Context, names []str
 	ctx, cancel := context.WithTimeout(ctx, k.params.waitTimeout())
 	defer cancel()
 
-	ai, err := k.statusAccessInformation(ctx, true)
+	ai, err := k.statusAccessInformation(ctx, true, true)
 	if err != nil {
 		return err
 	}
