@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// Copyright 2020-2022 Authors of Cilium
+// Copyright Authors of Cilium
 
 package check
 
@@ -7,6 +7,8 @@ import (
 	"bytes"
 	"context"
 	"fmt"
+	"net"
+	"strconv"
 	"strings"
 	"time"
 
@@ -16,6 +18,7 @@ import (
 	"golang.org/x/exp/slices"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/credentials/insecure"
+	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	"github.com/cilium/cilium-cli/defaults"
@@ -35,6 +38,8 @@ type ConnectivityTest struct {
 	// Parameters to the test suite, specified by the CLI user.
 	params Parameters
 
+	version string
+
 	// Clients for source and destination clusters.
 	clients *deploymentClients
 
@@ -47,10 +52,18 @@ type ConnectivityTest struct {
 	echoServices      map[string]Service
 	externalWorkloads map[string]ExternalWorkload
 
+	hostNetNSPodsByNode map[string]Pod
+
 	tests     []*Test
 	testNames map[string]struct{}
 
 	lastFlowTimestamps map[string]time.Time
+
+	nodes              map[string]*corev1.Node
+	nodesWithoutCilium []string
+
+	manifests      map[string]string
+	helmYAMLValues string
 }
 
 type PerfTests struct {
@@ -148,25 +161,28 @@ func (ct *ConnectivityTest) failedActions() []*Action {
 }
 
 // NewConnectivityTest returns a new ConnectivityTest.
-func NewConnectivityTest(client *k8s.Client, p Parameters) (*ConnectivityTest, error) {
+func NewConnectivityTest(client *k8s.Client, p Parameters, version string) (*ConnectivityTest, error) {
 	if err := p.validate(); err != nil {
 		return nil, err
 	}
 
 	k := &ConnectivityTest{
-		client:             client,
-		params:             p,
-		ciliumPods:         make(map[string]Pod),
-		echoPods:           make(map[string]Pod),
-		clientPods:         make(map[string]Pod),
-		perfClientPods:     make(map[string]Pod),
-		perfServerPod:      make(map[string]Pod),
-		PerfResults:        make(map[PerfTests]PerfResult),
-		echoServices:       make(map[string]Service),
-		externalWorkloads:  make(map[string]ExternalWorkload),
-		tests:              []*Test{},
-		testNames:          make(map[string]struct{}),
-		lastFlowTimestamps: make(map[string]time.Time),
+		client:              client,
+		params:              p,
+		version:             version,
+		ciliumPods:          make(map[string]Pod),
+		echoPods:            make(map[string]Pod),
+		clientPods:          make(map[string]Pod),
+		perfClientPods:      make(map[string]Pod),
+		perfServerPod:       make(map[string]Pod),
+		PerfResults:         make(map[PerfTests]PerfResult),
+		echoServices:        make(map[string]Service),
+		externalWorkloads:   make(map[string]ExternalWorkload),
+		hostNetNSPodsByNode: make(map[string]Pod),
+		nodes:               make(map[string]*corev1.Node),
+		tests:               []*Test{},
+		testNames:           make(map[string]struct{}),
+		lastFlowTimestamps:  make(map[string]time.Time),
 	}
 
 	return k, nil
@@ -224,6 +240,9 @@ func (ct *ConnectivityTest) SetupAndValidate(ctx context.Context) error {
 	if err := ct.detectFeatures(ctx); err != nil {
 		return err
 	}
+	if err := ct.getNodes(ctx); err != nil {
+		return err
+	}
 
 	if ct.debug() {
 		fs := make([]Feature, 0, len(ct.features))
@@ -262,6 +281,14 @@ func (ct *ConnectivityTest) Run(ctx context.Context) error {
 		return err
 	}
 
+	if len(ct.params.DeleteCiliumOnNodes) > 0 {
+		// Delete Cilium pods so only the datapath plumbing remains
+		ct.Debug("Deleting Cilium pods from specified nodes")
+		if err := ct.deleteCiliumPods(ctx); err != nil {
+			return err
+		}
+	}
+
 	ct.Debug("Registered connectivity tests:")
 	for _, t := range ct.tests {
 		ct.Debugf("  %s", t)
@@ -284,6 +311,9 @@ func (ct *ConnectivityTest) Run(ctx context.Context) error {
 			if err := t.Run(ctx); err != nil {
 				// We know for sure we're inside a separate goroutine, so Fatal()
 				// is safe and will properly record failure statistics.
+				if t.ctx.params.CollectSysdumpOnFailure {
+					t.collectSysdump()
+				}
 				t.Fatalf("Running test %s: %s", t.Name(), err)
 			}
 
@@ -466,7 +496,7 @@ func (ct *ConnectivityTest) initClients(ctx context.Context) error {
 // initCiliumPods fetches the Cilium agent pod information from all clients
 func (ct *ConnectivityTest) initCiliumPods(ctx context.Context) error {
 	for _, client := range ct.clients.clients() {
-		ciliumPods, err := client.ListPods(ctx, ct.params.CiliumNamespace, metav1.ListOptions{LabelSelector: "k8s-app=cilium"})
+		ciliumPods, err := client.ListPods(ctx, ct.params.CiliumNamespace, metav1.ListOptions{LabelSelector: ct.params.AgentPodSelector})
 		if err != nil {
 			return fmt.Errorf("unable to list Cilium pods: %w", err)
 		}
@@ -482,29 +512,105 @@ func (ct *ConnectivityTest) initCiliumPods(ctx context.Context) error {
 	return nil
 }
 
+func (ct *ConnectivityTest) getNodes(ctx context.Context) error {
+	nodeList, err := ct.client.ListNodes(ctx, metav1.ListOptions{})
+	if err != nil {
+		return fmt.Errorf("unable to list K8s Nodes: %w", err)
+	}
+
+	for _, node := range nodeList.Items {
+		node := node
+		if canNodeRunCilium(&node) {
+			ct.nodes[node.ObjectMeta.Name] = node.DeepCopy()
+		}
+	}
+
+	return nil
+}
+
 // DetectMinimumCiliumVersion returns the smallest Cilium version running in
 // the cluster(s)
 func (ct *ConnectivityTest) DetectMinimumCiliumVersion(ctx context.Context) (*semver.Version, error) {
 	var minVersion *semver.Version
 	for name, ciliumPod := range ct.ciliumPods {
-		stdout, err := ciliumPod.K8sClient.ExecInPodWithTTY(ctx, ciliumPod.Pod.Namespace, ciliumPod.Pod.Name,
-			defaults.AgentContainerName, []string{"cilium", "version", "-o", "jsonpath={$.Daemon.Version}"})
-		if err != nil {
-			return nil, fmt.Errorf("unable to fetch cilium version on pod %q: %w", name, err)
-		}
-		v, _, _ := strings.Cut(stdout.String(), "-") // strips proprietary -releaseX suffix
-		podVersion, err := semver.Parse(v)
+		podVersion, err := ciliumPod.K8sClient.GetCiliumVersion(ctx, ciliumPod.Pod)
 		if err != nil {
 			return nil, fmt.Errorf("unable to parse cilium version on pod %q: %w", name, err)
 		}
 		if minVersion == nil || podVersion.LT(*minVersion) {
-			minVersion = &podVersion
+			minVersion = podVersion
 		}
 	}
 
 	return minVersion, nil
 }
 
+// UninstallResources deletes all k8s resources created by the connectivity tests.
+func (ct *ConnectivityTest) UninstallResources(ctx context.Context, wait bool) {
+	ct.Logf("🔥 Deleting pods in %s namespace...", ct.params.TestNamespace)
+	ct.client.DeletePodCollection(ctx, ct.params.TestNamespace, metav1.DeleteOptions{}, metav1.ListOptions{})
+
+	ct.Logf("🔥 Deleting %s namespace...", ct.params.TestNamespace)
+	ct.client.DeleteNamespace(ctx, ct.params.TestNamespace, metav1.DeleteOptions{})
+
+	// To avoid cases where test pods are stuck in terminating state because
+	// cni (cilium) pods were deleted sooner, wait until test pods are deleted
+	// before moving onto deleting cilium pods.
+	if wait {
+		ct.Logf("⌛ Waiting for %s namespace to be terminated...", ct.params.TestNamespace)
+		for {
+			// Wait for the test namespace to be terminated. Subsequent connectivity checks would fail
+			// if the test namespace is in Terminating state.
+			_, err := ct.client.GetNamespace(ctx, ct.params.TestNamespace, metav1.GetOptions{})
+			if err == nil {
+				time.Sleep(defaults.WaitRetryInterval)
+			} else {
+				break
+			}
+		}
+	}
+}
+
+func (ct *ConnectivityTest) CurlCommand(peer TestPeer, ipFam IPFamily, opts ...string) []string {
+	cmd := []string{"curl",
+		"-w", "%{local_ip}:%{local_port} -> %{remote_ip}:%{remote_port} = %{response_code}",
+		"--silent", "--fail", "--show-error",
+		"--output", "/dev/null",
+	}
+
+	if connectTimeout := ct.params.ConnectTimeout.Seconds(); connectTimeout > 0.0 {
+		cmd = append(cmd, "--connect-timeout", strconv.FormatFloat(connectTimeout, 'f', -1, 64))
+	}
+	if requestTimeout := ct.params.RequestTimeout.Seconds(); requestTimeout > 0.0 {
+		cmd = append(cmd, "--max-time", strconv.FormatFloat(requestTimeout, 'f', -1, 64))
+	}
+
+	cmd = append(cmd, opts...)
+	cmd = append(cmd, fmt.Sprintf("%s://%s%s",
+		peer.Scheme(),
+		net.JoinHostPort(peer.Address(ipFam), fmt.Sprint(peer.Port())),
+		peer.Path()))
+	return cmd
+}
+
+func (ct *ConnectivityTest) PingCommand(peer TestPeer, ipFam IPFamily) []string {
+	cmd := []string{"ping", "-c", "1"}
+
+	if ipFam == IPFamilyV6 {
+		cmd = append(cmd, "-6")
+	}
+
+	if connectTimeout := ct.params.ConnectTimeout.Seconds(); connectTimeout > 0.0 {
+		cmd = append(cmd, "-W", strconv.FormatFloat(connectTimeout, 'f', -1, 64))
+	}
+	if requestTimeout := ct.params.RequestTimeout.Seconds(); requestTimeout > 0.0 {
+		cmd = append(cmd, "-w", strconv.FormatFloat(requestTimeout, 'f', -1, 64))
+	}
+
+	cmd = append(cmd, peer.Address(ipFam))
+	return cmd
+}
+
 func (ct *ConnectivityTest) RandomClientPod() *Pod {
 	for _, p := range ct.clientPods {
 		return &p
@@ -520,10 +626,18 @@ func (ct *ConnectivityTest) CiliumPods() map[string]Pod {
 	return ct.ciliumPods
 }
 
+func (ct *ConnectivityTest) Nodes() map[string]*corev1.Node {
+	return ct.nodes
+}
+
 func (ct *ConnectivityTest) ClientPods() map[string]Pod {
 	return ct.clientPods
 }
 
+func (ct *ConnectivityTest) HostNetNSPodsByNode() map[string]Pod {
+	return ct.hostNetNSPodsByNode
+}
+
 func (ct *ConnectivityTest) PerfServerPod() map[string]Pod {
 	return ct.perfServerPod
 }
@@ -567,3 +681,12 @@ func (ct *ConnectivityTest) PostTestSleepDuration() time.Duration {
 func (ct *ConnectivityTest) K8sClient() *k8s.Client {
 	return ct.client
 }
+
+func (ct *ConnectivityTest) NodesWithoutCilium() []string {
+	return ct.nodesWithoutCilium
+}
+
+func (ct *ConnectivityTest) Feature(f Feature) (FeatureStatus, bool) {
+	s, ok := ct.features[f]
+	return s, ok
+}
