@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// Copyright 2021 Authors of Cilium
+// Copyright Authors of Cilium
 
 package tests
 
@@ -25,17 +25,18 @@ func (s *podToCIDR) Name() string {
 
 func (s *podToCIDR) Run(ctx context.Context, t *check.Test) {
 	eps := []check.TestPeer{
-		check.HTTPEndpoint("cloudflare-1001", "http://1.0.0.1"),
-		check.HTTPEndpoint("cloudflare-1111", "http://1.1.1.1"),
+		check.HTTPEndpoint("cloudflare-1001", "https://"+t.Context().Params().ExternalOtherIP),
+		check.HTTPEndpoint("cloudflare-1111", "https://"+t.Context().Params().ExternalIP),
 	}
+	ct := t.Context()
 
 	for _, ep := range eps {
 		var i int
-		for _, src := range t.Context().ClientPods() {
+		for _, src := range ct.ClientPods() {
 			src := src // copy to avoid memory aliasing when using reference
 
-			t.NewAction(s, fmt.Sprintf("%s-%d", ep.Name(), i), &src, ep).Run(func(a *check.Action) {
-				a.ExecInPod(ctx, curl(ep))
+			t.NewAction(s, fmt.Sprintf("%s-%d", ep.Name(), i), &src, ep, check.IPFamilyNone).Run(func(a *check.Action) {
+				a.ExecInPod(ctx, ct.CurlCommand(ep, check.IPFamilyNone))
 
 				a.ValidateFlows(ctx, src, a.GetEgressRequirements(check.FlowParameters{
 					RSTAllowed: true,
