@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// Copyright 2021 Authors of Cilium
+// Copyright Authors of Cilium
 
 package tests
 
@@ -22,13 +22,14 @@ func (s *podToExternalWorkload) Name() string {
 
 func (s *podToExternalWorkload) Run(ctx context.Context, t *check.Test) {
 	var i int
+	ct := t.Context()
 
-	for _, pod := range t.Context().ClientPods() {
+	for _, pod := range ct.ClientPods() {
 		pod := pod // copy to avoid memory aliasing when using reference
 
-		for _, wl := range t.Context().ExternalWorkloads() {
-			t.NewAction(s, fmt.Sprintf("ping-%d", i), &pod, wl).Run(func(a *check.Action) {
-				a.ExecInPod(ctx, ping(wl))
+		for _, wl := range ct.ExternalWorkloads() {
+			t.NewAction(s, fmt.Sprintf("ping-%d", i), &pod, wl, check.IPFamilyV4).Run(func(a *check.Action) {
+				a.ExecInPod(ctx, ct.PingCommand(wl, check.IPFamilyV4))
 
 				a.ValidateFlows(ctx, pod, a.GetEgressRequirements(check.FlowParameters{
 					Protocol: check.ICMP,
