@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// Copyright 2020-2021 Authors of Cilium
+// Copyright Authors of Cilium
 
 package install
 
@@ -8,6 +8,7 @@ import (
 	"context"
 	"fmt"
 	"io"
+	"regexp"
 	"strings"
 	"time"
 
@@ -18,6 +19,7 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	networkingv1 "k8s.io/api/networking/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
@@ -109,7 +111,7 @@ func (k *K8sInstaller) generateIngressClass() *networkingv1.IngressClass {
 
 	switch {
 	case versioncheck.MustCompile(">=1.12.0")(k.chartVersion):
-		ingressFileName = "templates/cilium-operator-deployment.yaml"
+		ingressFileName = "templates/cilium-ingress-class.yaml"
 	}
 
 	ingressClassFile, exists := k.manifests[ingressFileName]
@@ -122,6 +124,26 @@ func (k *K8sInstaller) generateIngressClass() *networkingv1.IngressClass {
 	return &ingressClass
 }
 
+func (k *K8sInstaller) generateIngressService() *corev1.Service {
+	var (
+		ingressServiceFilename string
+	)
+
+	switch {
+	case versioncheck.MustCompile(">=1.13.0")(k.chartVersion):
+		ingressServiceFilename = "templates/cilium-ingress-service.yaml"
+	}
+
+	ingressServiceFile, exists := k.manifests[ingressServiceFilename]
+	if !exists {
+		return nil
+	}
+
+	var ingressService corev1.Service
+	utils.MustUnmarshalYAML([]byte(ingressServiceFile), &ingressService)
+	return &ingressService
+}
+
 func (k *K8sInstaller) getSecretNamespace() string {
 	var (
 		nsFilename string
@@ -144,7 +166,9 @@ func (k *K8sInstaller) getSecretNamespace() string {
 
 type k8sInstallerImplementation interface {
 	ClusterName() string
+	GetAPIServerHostAndPort() (string, string)
 	ListNodes(ctx context.Context, options metav1.ListOptions) (*corev1.NodeList, error)
+	PatchNode(ctx context.Context, nodeName string, pt types.PatchType, data []byte) (*corev1.Node, error)
 	GetCiliumExternalWorkload(ctx context.Context, name string, opts metav1.GetOptions) (*ciliumv2.CiliumExternalWorkload, error)
 	CreateCiliumExternalWorkload(ctx context.Context, cew *ciliumv2.CiliumExternalWorkload, opts metav1.CreateOptions) (*ciliumv2.CiliumExternalWorkload, error)
 	DeleteCiliumExternalWorkload(ctx context.Context, name string, opts metav1.DeleteOptions) error
@@ -159,14 +183,18 @@ type k8sInstallerImplementation interface {
 	CreateClusterRoleBinding(ctx context.Context, role *rbacv1.ClusterRoleBinding, opts metav1.CreateOptions) (*rbacv1.ClusterRoleBinding, error)
 	DeleteClusterRoleBinding(ctx context.Context, name string, opts metav1.DeleteOptions) error
 	CreateRole(ctx context.Context, namespace string, role *rbacv1.Role, opts metav1.CreateOptions) (*rbacv1.Role, error)
+	UpdateRole(ctx context.Context, namespace string, role *rbacv1.Role, opts metav1.UpdateOptions) (*rbacv1.Role, error)
 	DeleteRole(ctx context.Context, namespace string, name string, opts metav1.DeleteOptions) error
 	CreateRoleBinding(ctx context.Context, namespace string, roleBinding *rbacv1.RoleBinding, opts metav1.CreateOptions) (*rbacv1.RoleBinding, error)
+	UpdateRoleBinding(ctx context.Context, namespace string, roleBinding *rbacv1.RoleBinding, opts metav1.UpdateOptions) (*rbacv1.RoleBinding, error)
 	DeleteRoleBinding(ctx context.Context, namespace, name string, opts metav1.DeleteOptions) error
 	CreateDaemonSet(ctx context.Context, namespace string, ds *appsv1.DaemonSet, opts metav1.CreateOptions) (*appsv1.DaemonSet, error)
+	ListDaemonSet(ctx context.Context, namespace string, o metav1.ListOptions) (*appsv1.DaemonSetList, error)
 	GetDaemonSet(ctx context.Context, namespace, name string, opts metav1.GetOptions) (*appsv1.DaemonSet, error)
 	DeleteDaemonSet(ctx context.Context, namespace, name string, opts metav1.DeleteOptions) error
 	PatchDaemonSet(ctx context.Context, namespace, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions) (*appsv1.DaemonSet, error)
 	GetService(ctx context.Context, namespace, name string, opts metav1.GetOptions) (*corev1.Service, error)
+	GetEndpoints(ctx context.Context, namespace, name string, opts metav1.GetOptions) (*corev1.Endpoints, error)
 	CreateService(ctx context.Context, namespace string, service *corev1.Service, opts metav1.CreateOptions) (*corev1.Service, error)
 	DeleteService(ctx context.Context, namespace, name string, opts metav1.DeleteOptions) error
 	DeleteDeployment(ctx context.Context, namespace, name string, opts metav1.DeleteOptions) error
@@ -196,6 +224,9 @@ type k8sInstallerImplementation interface {
 	GetServerVersion() (*semver.Version, error)
 	CreateIngressClass(ctx context.Context, r *networkingv1.IngressClass, opts metav1.CreateOptions) (*networkingv1.IngressClass, error)
 	DeleteIngressClass(ctx context.Context, name string, opts metav1.DeleteOptions) error
+	CiliumLogs(ctx context.Context, namespace, pod string, since time.Time, filter *regexp.Regexp) (string, error)
+	ListAPIResources(ctx context.Context) ([]string, error)
+	GetHelmState(ctx context.Context, namespace string, secretName string) (*helm.State, error)
 }
 
 type K8sInstaller struct {
@@ -303,6 +334,15 @@ type Parameters struct {
 
 	// ListVersions lists all the available versions for install without actually installing.
 	ListVersions bool
+
+	// NodesWithoutCilium lists all nodes on which Cilium is not installed.
+	NodesWithoutCilium []string
+
+	// APIVersions defines extra kubernetes api resources that can be passed to helm for capabilities validation,
+	// specifically for CRDs.
+	APIVersions []string
+	// UserSetKubeProxyReplacement will be set as true if user passes helm opt or commadline flag for the Kube-Proxy replacement.
+	UserSetKubeProxyReplacement bool
 }
 
 type rollbackStep func(context.Context)
@@ -312,7 +352,7 @@ func (p *Parameters) validate() error {
 	for _, config := range p.ConfigOverwrites {
 		t := strings.SplitN(config, "=", 2)
 		if len(t) != 2 {
-			return fmt.Errorf("invalid config overwrite %q, must be in the form key=valye", config)
+			return fmt.Errorf("invalid config overwrite %q, must be in the form key=value", config)
 		}
 
 		p.configOverwrites[t[0]] = t[1]
@@ -407,7 +447,7 @@ func (k *K8sInstaller) generateConfigMap() (*corev1.ConfigMap, error) {
 	k.Log("🚀 Creating ConfigMap for Cilium version %s...", k.chartVersion)
 
 	for key, value := range k.params.configOverwrites {
-		k.Log("ℹ️ Manual overwrite in ConfigMap: %s=%s", key, value)
+		k.Log("ℹ️  Manual overwrite in ConfigMap: %s=%s", key, value)
 		cm.Data[key] = value
 	}
 
@@ -614,6 +654,12 @@ func (k *K8sInstaller) Install(ctx context.Context) error {
 	if err != nil {
 		return err
 	}
+
+	if k.params.HelmGenValuesFile != "" {
+		k.Log("ℹ️  Generated helm values file %q successfully written", k.params.HelmGenValuesFile)
+		return nil
+	}
+
 	k.Log("ℹ️  Storing helm values file in %s/%s Secret", k.params.Namespace, k.params.HelmValuesSecretName)
 
 	helmSecret := k8s.NewSecret(k.params.HelmValuesSecretName, k.params.Namespace,
@@ -633,11 +679,6 @@ func (k *K8sInstaller) Install(ctx context.Context) error {
 		}
 	}
 
-	if k.params.HelmGenValuesFile != "" {
-		k.Log("ℹ️  Generated helm values file %q successfully written", k.params.HelmGenValuesFile)
-		return nil
-	}
-
 	switch k.flavor.Kind {
 	case k8s.KindEKS:
 		cm, err := k.generateConfigMap()
@@ -676,6 +717,16 @@ func (k *K8sInstaller) Install(ctx context.Context) error {
 		return err
 	}
 
+	for _, nodeName := range k.params.NodesWithoutCilium {
+		k.Log("🚀 Setting label %q on node %q to prevent Cilium from being scheduled on it...", defaults.CiliumNoScheduleLabel, nodeName)
+		label := utils.EscapeJSONPatchString(defaults.CiliumNoScheduleLabel)
+		labelPatch := fmt.Sprintf(`[{"op":"add","path":"/metadata/labels/%s","value":"true"}]`, label)
+		_, err = k.client.PatchNode(ctx, nodeName, types.JSONPatchType, []byte(labelPatch))
+		if err != nil {
+			return err
+		}
+	}
+
 	k.Log("🚀 Creating Service accounts...")
 	if _, err := k.client.CreateServiceAccount(ctx, k.params.Namespace, k.NewServiceAccount(defaults.AgentServiceAccountName), metav1.CreateOptions{}); err != nil {
 		return err
@@ -751,6 +802,18 @@ func (k *K8sInstaller) Install(ctx context.Context) error {
 		})
 	}
 
+	ingressService := k.generateIngressService()
+	if ingressService != nil {
+		if _, err := k.client.CreateService(ctx, ingressService.GetNamespace(), ingressService, metav1.CreateOptions{}); err != nil {
+			return err
+		}
+		k.pushRollbackStep(func(ctx context.Context) {
+			if err := k.client.DeleteService(ctx, ingressService.GetNamespace(), ingressService.GetName(), metav1.DeleteOptions{}); err != nil {
+				k.Log("Cannot delete %s Ingress Service: %s.%s", ingressService.GetNamespace(), ingressService.GetName(), err)
+			}
+		})
+	}
+
 	secretsNamespace := k.getSecretNamespace()
 	if len(secretsNamespace) != 0 {
 		if _, err := k.client.CreateNamespace(ctx, secretsNamespace, metav1.CreateOptions{}); err != nil {
@@ -761,31 +824,40 @@ func (k *K8sInstaller) Install(ctx context.Context) error {
 				k.Log("Cannot delete %s Namespace: %s", secretsNamespace, err)
 			}
 		})
+	}
 
-		for _, roleName := range []string{defaults.AgentSecretsRoleName, defaults.OperatorSecretsRoleName} {
-			r := k.NewRole(roleName)
-			if r == nil {
-				continue
+	for _, roleName := range []string{defaults.AgentSecretsRoleName, defaults.OperatorSecretsRoleName} {
+		rs := k.NewRole(roleName)
+
+		for _, r := range rs {
+			_, err = k.client.CreateRole(ctx, r.GetNamespace(), r, metav1.CreateOptions{})
+			if apierrors.IsAlreadyExists(err) {
+				_, err = k.client.UpdateRole(ctx, r.GetNamespace(), r, metav1.UpdateOptions{})
 			}
 
-			_, err = k.client.CreateRole(ctx, secretsNamespace, r, metav1.CreateOptions{})
 			if err != nil {
 				return err
 			}
 
 			k.pushRollbackStep(func(ctx context.Context) {
-				if err := k.client.DeleteRole(ctx, secretsNamespace, r.GetName(), metav1.DeleteOptions{}); err != nil {
+				if err := k.client.DeleteRole(ctx, r.GetNamespace(), r.GetName(), metav1.DeleteOptions{}); err != nil {
 					k.Log("Cannot delete %s Role: %s", r.GetName(), err)
 				}
 			})
+		}
 
-			rb, err := k.client.CreateRoleBinding(ctx, secretsNamespace, k.NewRoleBinding(roleName), metav1.CreateOptions{})
+		rbs := k.NewRoleBinding(roleName)
+		for _, rb := range rbs {
+			_, err := k.client.CreateRoleBinding(ctx, rb.GetNamespace(), rb, metav1.CreateOptions{})
+			if apierrors.IsAlreadyExists(err) {
+				_, err = k.client.UpdateRoleBinding(ctx, rb.GetNamespace(), rb, metav1.UpdateOptions{})
+			}
 			if err != nil {
 				return err
 			}
 			k.pushRollbackStep(func(ctx context.Context) {
-				if err := k.client.DeleteRoleBinding(ctx, secretsNamespace, rb.GetName(), metav1.DeleteOptions{}); err != nil {
-					k.Log("Cannot delete %s RoleBinding: %s", rb.GetName(), err)
+				if err := k.client.DeleteRoleBinding(ctx, rb.GetNamespace(), rb.GetName(), metav1.DeleteOptions{}); err != nil {
+					k.Log("Cannot delete %s RoleBinding: %s/%s", rb.GetNamespace(), rb.GetName(), err)
 				}
 			})
 		}
